defstar {
	name { Xscope }
	domain { SDF }
	desc {
Generate a multi-trace plot with the xgraph program.
Successive traces are overlaid on one another.
	}
	derivedFrom { Xgraph }
	version {@(#)SDFXscope.pl	2.11 2/12/96}
	author { J. T. Buck, N. Becker }
	copyright {
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
	}
	location { SDF main library }
	explanation {
This star is an enhanced version of Xgraph.  It is identical
except that it can plot multiple traces, like an oscilloscope.
As for Xgraph,
the \fItitle\fR parameter specifies a title for the plot.
The \fIsaveFile\fR parameter optionally specifies a file for
storing the data in a syntax acceptable to xgraph.
A null string prevents any such storage.
The \fIoptions\fR string is passed directly to the xgraph program
as command-line options.  See the manual section describing xgraph
for a complete explanation of the options.
.pp
Multiple traces may be plotted by setting the \fItraceLength\fR
state to a nonzero value.  In this case, a new plot (starting at
x value zero) is started every \fItraceLength\fR samples.  The
first \fIignore\fR samples are not plotted; this is useful for letting
transients die away.  The \fIinitCount\fR parameter can be used to
offset the traces to control their alignment with respect to the
edge of the plot.
.Ir "xgraph program"
.Id "oscilloscope, X window"
.Id "graph, X window, multi-trace"
	}
	defstate {
		name {traceLength}
		type {int}
		default {"0"}
		desc { Number of samples per trace.  If 0, only one trace. }
	}
	defstate {
		name {initCount}
		type {int}
		default {"0"}
		desc { Offset the traces }
	}
	protected {
		int traceCount;
		int nTracesSoFar;
	}

	setup {
		SDFXgraph::setup();
		nTracesSoFar = 0;
		traceCount = int(initCount);
	}

	go {
		if (int(traceLength) > 0 && traceCount >= int(traceLength)) {
			traceCount = 0;
			graph.newTrace();
			index = xInit;	// reinitialize x index
			nTracesSoFar++;
		}
		traceCount++;
		SDFXgraph::go();
	}
}
