static const char file_id[] = "DataStruct.cc";
/**************************************************************************
Version identification:
@(#)DataStruct.cc	2.20	2/27/96

Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

						PT_COPYRIGHT_VERSION_2
						COPYRIGHTENDKEY

 Programmer:  D. G. Messerschmitt, J. Buck and Jose Luis Pino
 Date of creation: 7/30/90

These are methods used by classes defined in DataStruct.h that
are too large to efficiently inline (formerly they were declared
inline anyway).

**************************************************************************/
#ifdef __GNUG__
#pragma implementation
#endif

#include "DataStruct.h"

SingleLink::SingleLink(Pointer a,SingleLink* p) {
    e=a;
    if (p) {
	next = p;
	previous = p->previous;
	p->previous = this;
	previous->next = this;
    }
    else {
	previous = this;
	next = this;
    }
}

void SingleLink::remove() {
    next->previous = previous;
    previous->next = next;
}

SingleLink* SequentialList::removeLink(SingleLink& a) {
    SingleLink* previous = a.previous;
    if (lastNode == &a) lastNode = a.previous;
    a.remove();
    delete &a;
    dimen--;
    if (dimen == 0) {
	lastNode = 0;
	previous = 0;
    }
    return previous;
}

// constructor with argument
SequentialList :: SequentialList(Pointer a) : dimen(1)
{
	LOG_NEW; lastNode= new SingleLink(a,0);
}

// add at head of list
void SequentialList :: prepend(Pointer a)
{
	if (dimen > 0) {	// List not empty
	    LOG_NEW; new SingleLink(a,lastNode->next);
	}
	else {	               // List empty
	    LOG_NEW; lastNode = new SingleLink(a,0);
	}
	dimen++;
}

// add at tail of list
void SequentialList :: append(Pointer a)
{
	if (dimen > 0) {	// List not empty
	    LOG_NEW; lastNode = new SingleLink(a,lastNode->next);
	}
	else {		        // List empty
	    LOG_NEW; lastNode = new SingleLink(a,0);
	}
	dimen++;
}

// return and remove head of list
Pointer SequentialList :: getAndRemove()
{
	if (dimen == 0) return 0;

	SingleLink *f = lastNode->next;	// Head of list
	Pointer r = f->e;

	removeLink(*f);
	
	return r;
}

// return and remove tail of list
Pointer SequentialList :: getTailAndRemove()
{
	if (dimen == 0) return 0;

        Pointer r = lastNode->e;        // Save contents of tail

	removeLink(*lastNode);
	
        return r;
}

// return i-th element of list, null if fewer than i elements.
Pointer SequentialList :: elem(int i) const
{
	if (i > dimen) return 0;
	SingleLink *f = lastNode->next;	// Head of list
	for( int t = i; t > 0; t-- )
		f = f->next;
	return f->e;
}

void SequentialList :: initialize()
{
	if (empty()) return;	// List already empty

	// Point to the first element in the list
	SingleLink *l = lastNode->next;

	// As long as the first element is not also the last, delete it
	while (l != lastNode ) {
		SingleLink *ll=l;
		l = l->next;
		LOG_DEL; delete ll;
	}

	// Delete the last node in the list
	LOG_DEL; delete lastNode;

	// and mark the list empty
	lastNode = 0;
	dimen = 0;
}

// This function searches for an element in a SequentialList matching
// the argument, removing it if found.
int SequentialList::remove (Pointer x) {
	// case of empty list
	if (dimen == 0) return 0;
	// case of 1-element list
	if (dimen == 1) {
		if (lastNode->e != x) return 0;
		// only element matches, zero the list
		LOG_DEL; delete lastNode;
		lastNode = 0;
		dimen = 0;
		return 1;
	}
	// general case
	SingleLink* f = lastNode->next;
	do {
		if (f->e == x) {
		    removeLink(*f);
		    return 1;
		}
		f = f->next;
	} while (f != lastNode->next);
	return 0;
}

// This function returns true if the argument is found in the list.
int SequentialList::member (Pointer x) const {
	if (dimen == 0) return 0;
	SingleLink* f = lastNode;
	do {
		if (f->e == x) return TRUE;
		f = f->next;
	} while (f != lastNode);
	return FALSE;
}

void ListIter::reset() {
    startAtHead = TRUE;
    ref = 0;
}

// This routine has been re-implemented and optimized for speed
// because of its heavy usage.  The if-structure is organized
// so that necessary ifs are executed, but for cases where only
// a few ifs are needed, the minimum number of ifs is done for
// the most common cases, with rarer cases taking decreasing
// priority in the if-structure.
Pointer ListIter::next() {
  // Necessary check, distinguishes fundamentally different
  // behavior.  Else-branch taken exactly once when next called
  // after ListIter creation or after ListIter reset.
  if (!startAtHead) {
    // Next rarer case, check for ListIter reaching end of list.
    // Else-branch taken once per list traversal, if allowed to
    // traverse to the end.
    if (ref != list->lastNode) {
      // Next rarer case, check for null ref.  Happens if next
      // called after reaching end of list or after a next call
      // where null list->lastNode is found.
      if (ref) {
	// Rarest case, check for null list->lastNode.  Must
	// pass through all if-checks in order to finally reach
	// code to step forward in list.  Need to know that there
	// is an end and that traversal will ultimately terminate.
        if (list->lastNode) {
	  ref = ref->next;
	  return ref->e;
	}
	// Null list->lastNode.
	else {
	  ref = 0;
	  return 0;
	}
      }
      // Null ref.
      else {
	return 0;
      }
    }
    // End of list reached, ref == list->lastNode.
    else {
      ref = 0;
      return 0;
    }
  }
  // Starting at the head of the list, so enter the list at the
  // node pointed to by list->lastNode->next.
  else {
    startAtHead = FALSE;
    // Rarest case, check for null list->lastNode.  Must
    // pass through all if-checks in order to finally reach
    // code to step forward in list.  Need to know that there
    // is an end and that traversal will ultimately terminate.
    // Also, in this case, hedges against dereferencing a
    // null list->lastNode.
    if (list->lastNode) {
      ref = list->lastNode->next;
      return ref->e;
    }
    // Null list->lastNode.
    else {
      ref = 0;
      return 0;
    }
  }
  // Should never be reached.  Included for safety in case
  // preceding if-structure is broken by future changes.
  return 0;
}

void ListIter::reconnect(const SequentialList& l) {
    list = &l;
    reset();
}

void ListIter::remove() {
    if (ref) {
	ref = ((SequentialList*)list)->removeLink(*ref);
	if (ref == list->lastNode) startAtHead = TRUE;
    }
    else if (list->lastNode)
	((SequentialList*)list)->removeLink(*list->lastNode);
}
