# misc.itcl - Miscellaneous utility functions.
#
##########################################################################
#
# Author:  H. John Reekie & Jose Luis Pino
#
# Version: @(#)misc.tcl	1.10 2/22/96
#
# Copyright (c) 1990-1996 The Regents of the University of California.
# All rights reserved.
# 
# Permission is hereby granted, without written agreement and without
# license or royalty fees, to use, copy, modify, and distribute this
# software and its documentation for any purpose, provided that the
# above copyright notice and the following two paragraphs appear in all
# copies of this software.
# 
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
# THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# 
# THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
# CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.
# 
# 						PT_COPYRIGHT_VERSION_2
# 						COPYRIGHTENDKEY
##########################################################################

##########################################################################
##### null 
# Test for null object. Within the expression parser, this is easily
# enough done using $x == "". However, it is sometimes convenient
# to use this routine, as it saves writing [expr {$x == ""}]. (The
# curly braces are necessary.)
# 
# Return: 1 if x is null, 0 otherwise.
#
proc null {x} {
    return [expr {$x == ""}]
}


##########################################################################
##### notnull 
# Test if an object is not a null object. 
#
# Return: 1 if x is not null, 0 otherwise.
#
proc notnull {x} {
    return [expr {$x != ""}]
}


##########################################################################
##### isNonNegativeInt 
# This procedure determines whether or not n is a non-negative integer.
#
# Return: 1 if x is a non-negative integer, 0 otherwise.
#
proc isNonNegativeInt {n} {
    return [regexp {^[0-9]+$} $n]
}


##########################################################################
##### isInteger
# This procedure determines whether or not n is an integer.
#
# Return: 1 if x is an integer, 0 otherwise.
#
proc isInteger {n} {
    return [regexp {^-?[0-9]+$} $n]
}


##########################################################################
##### isNumber
# This procedure determines whether or not n is an integer or a
# decimal number. Note that it does not accept exponential
# notation.
#
# Return: 1 if x is a number, 0 otherwise.
#
proc isNumber {n} {
    return [regexp {^-?[0-9]+\.?[0-9]*$} $n]
}


##########################################################################
#### isPathname name
#
# Test if a name is an itcl path, beginning with "::". This can
# probably be used in many situations when isObject{} is used,
# and may be more efficient.
#
proc isPathname {name} {
    return [string match {::*} $name]
}


##########################################################################
#### isObject name
#
# Test if a name represents an itcl object.
#
proc isObject {name} {
    return [expr {[info object $name] != ""}]
}


##########################################################################
#### isClass name
#
# Test if a name represents an itcl class.
#
proc isClass {name} {
    return [expr {[info class $name] != ""}]
}


##########################################################################
#### isCanvasItem name
#
# Test if a name represents a Tk canvas item.
#
proc isCanvasItem {name} {
    return [isInteger $name]
}
